#include "osHandles.h"              // Includes all FreeRTOS Files

#include "System/cpu.h"             // Initialize CPU Hardware
#include "drivers/watchdog.h"		// Watchdog startup delay
#include "drivers/uart0.h"       	// Initialize UART
#include "utils/rprintf.h"			// Reduced printf.  STDIO uses lot of FLASH space & Stack space
#include "fat/diskio.h"				// disk_timerproc()
#include "tasks/shift_brite.h"
#include "tasks/userInterface.hpp"	// User interface functions to interact through UART
#include "utils/timer.hpp"			// FreeRTOS based Timer class
#include "drivers/motor.hpp"
// TODO 0.   In Eclipse, go to "Tasks" and complete the TODOs to input your SPI driver and read SD Card File System



/// This function is required for File System Disk Timout that should run every 10ms
void FatFsDiskTimer(void* p)
{
	disk_timerproc();
}
void shift_brite(void *p)
{
	OSHANDLES *osHandles = (OSHANDLES*) p; //what is this?
	PINSEL0 &= ~(0xFF<<8); //P0.4 to P0.7 to GPIO

	//P0.4 to P0.7 to OUTPUT
	IODIR0 |= (1<<4);
	IODIR0 |= (1<<5);
	IODIR0 |= (1<<6);
	IODIR0 |= (1<<7);
	//set the enable output (P0.6) low to enable the shiftbrite
	IOCLR0 = (1<<6);
	//IOCLR0 = (1<<5);
	latchLow();
	rprintf("shiftbrite is initialized\n");
	  while(1)
	    {
		  latchHigh();
		  sendPacket(colorPacket(0,0,1000));
		  //sendPacket(colorPacket(200,0,0));
		  latchLow();

		  /*sendPacket(colorPacket(0,0,200));
		  vTaskDelay(400);
		  latch();
		  sendPacket(colorPacket(0,200,200));
		  vTaskDelay(400);
		  latch();
		  sendPacket(colorPacket(200,0,200));
		  vTaskDelay(400);
		  latch();*/

		  /*vTaskDelay(1000);
		  sendPacket(colorPacket(0,1023,0));
		  vTaskDelay(1000);
		  sendPacket(colorPacket(0,0,1023));
		  vTaskDelay(1000);
		  sendPacket(colorPacket(1023,0,1023));
		  vTaskDelay(1000);*/
	    }
	  while (1);

}


/* INTERRUPT VECTORS:
 * 0:    OS Timer Tick
 * 1:    Not Used
 * 2:    UART0 Interrupt
 * 3:    Not Used
 * 4:    Not Used
 * 5-16: Not Used
 */
int main (void)
{
	OSHANDLES SysHandles;        // Should contain all OS Handles

	cpuSetupHardware();          // Setup PLL, enable MAM etc.
	watchdogDelayUs(1000*1000);  // Some startup delay
	uart0Init(38400, 64);        // 64 is size of Rx/Tx FIFO

	// Use polling version of uart0 to do printf/rprintf before starting FreeRTOS
	rprintf_devopen(uart0PutCharPolling);
	cpuPrintMemoryInfo();

	// Open interrupt-driven version of UART0 Rx/Tx
	rprintf_devopen(uart0PutChar);
	watchdogDelayUs(1000*1000);

	/** Create timer needed for SD Card I/O */
	Timer diskTimer(FatFsDiskTimer, 10, TimerPeriodic);
	diskTimer.start();

	/** Create any Queues and Mutexes **/
	SysHandles.lock.SPI = xSemaphoreCreateMutex();


	// Use the WATERMARK command to determine optimal Stack size of each task (set to high, then slowly decrease)
	// Priorities should be set according to response required
	if (
		pdPASS != xTaskCreate( uartUI, (signed char*)"Uart UI", STACK_BYTES(1024*6), &SysHandles, PRIORITY_LOW, &SysHandles.task.userInterface )
		//||
		//pdPASS != xTaskCreate( motor, (signed char*)"motor", STACK_BYTES(1024), &SysHandles, PRIORITY_LOW, &SysHandles.task.motor )
		//||
		//pdPASS != xTaskCreate( shift_brite, (signed char*)"shift_brite", STACK_BYTES(1024*6), &SysHandles, PRIORITY_LOW, &SysHandles.task.shift_brite )
	){
		rprintf("ERROR:  OUT OF MEMORY: Check OS Stack Size or task stack size.\n");
	}

	// Start FreeRTOS to begin servicing tasks created above, vTaskStartScheduler() will not "return"
	rprintf("\n-- Starting FreeRTOS --\n");
	vTaskStartScheduler();

	// In case OS is exited:
	rprintf_devopen(uart0PutCharPolling);
	rprintf("ERROR: Unexpected OS Exit!\n");

	return 0;
}

