/*
 * shift_brite.c
 *
 *  Created on: Apr 26, 2012
 *      Author: Thinh Duong
 */

#include "shift_brite.h"
#include "../System/lpc214x.h"
#include "utils/rprintf.h"     // rprintf

#include <stdlib.h>

// colorPacket returns a ShiftBritePacket for setting color brightnesses
//
// red, green, and blue are brightness values from 0 to 1023. 0 is off, and
// 1023 is brightest.
ShiftBritePacket colorPacket(unsigned int red, unsigned int green, unsigned int blue)
{
	//rprintf("colorPACKET red %X, green %X, blue %X\n",red, green, green);
    //Make a packet and initialize all of the bits to zero.
    ShiftBritePacket shiftbrite_packet = {value:0};

    shiftbrite_packet.red   = red;
    shiftbrite_packet.green = green;
    shiftbrite_packet.blue  = blue;

    return shiftbrite_packet;
}

// commandPacket returns a ShiftBritePacket for sending commands to the A6281.
//
// redDotCorrect, greenDotCorrect, and blueDotCorrect lets you control what
//   percentage of current is flowing to each color diode.
//   Refer to page 8 of the datasheet for more information.
// clockMode lets you set the PWM frequency for the diodes.
//   Refer to page 7 of the datasheet for more information.
ShiftBritePacket commandPacket(unsigned int redDotCorrect, unsigned int greenDotCorrect,
                               unsigned int blueDotCorrect, unsigned char clockMode)
{
    //Make a packet and initialize all of the bits to zero.
    ShiftBritePacket shiftbrite_packet = {value:0};

    shiftbrite_packet.redDotCorrect   = redDotCorrect;
    shiftbrite_packet.greenDotCorrect = greenDotCorrect;
    shiftbrite_packet.blueDotCorrect  = blueDotCorrect;
    shiftbrite_packet.clockMode = clockMode;
    shiftbrite_packet.command = 1;

    return shiftbrite_packet;
}

void sendPacket(ShiftBritePacket shiftbrite_packet)
{
	rprintf("SENDPACKET red %i, green %i, blue %i\n",shiftbrite_packet.red, shiftbrite_packet.green, shiftbrite_packet.green);
	int i = 0;
    for(i = 1; i < 32 + 1; i++)
    {
        //Set the appropriate Data In value according to the packet.
        if ((shiftbrite_packet.value >> (32 - i)) & 1)
            //PORTC |= (1 << PORTC2);
        	IOSET0 = (1<<4);
        else
            //PORTC &= ~(1 << PORTC2);
        	IOCLR = (1<<4);

        //Toggle the clock bit twice.
        //PORTC ^= (1 << PORTC5);
        //PORTC ^= (1 << PORTC5);
        IOPIN ^= (1<<7);
        IOPIN ^= (1<<7);


    }
    rprintf("done sending 32bits\n");
}
void latch()
{
	//rprintf("Setting latch HIGH\n");
	    // Set Latch high
	    //PORTC |= (1 << PORTC3);
		IOSET0 = (1<<5);
	    // Set Latch low
	    //PORTC &= ~(1 << PORTC3);
		//rprintf("Setting latch LOW\n");
		IOCLR0 = (1<<5);
}
void latchHigh()
{
	rprintf("Setting latch HIGH\n");
    // Set Latch high
    //PORTC |= (1 << PORTC3);
	IOSET0 = (1<<5);
    // Set Latch low
    //PORTC &= ~(1 << PORTC3);
	//rprintf("Setting latch LOW\n");
	//IOCLR0 = (1<<5);
}

void latchLow()
{
	rprintf("Setting latch LOW\n");
	IOCLR0 = (1<<5);

}
void sendColorWheelPacket(unsigned int location_on_wheel)
{
	rprintf("sendcolorpacket\n");
    if (location_on_wheel < 400)
        sendPacket(colorPacket(400,(location_on_wheel % 400),0));
    else if (location_on_wheel < 800)
        sendPacket(colorPacket(400-(location_on_wheel % 400),400,0));
    else if (location_on_wheel < 1200)
        sendPacket(colorPacket(0,400,(location_on_wheel % 400)));
    else if (location_on_wheel < 1600)
        sendPacket(colorPacket(0,400-(location_on_wheel % 400),400));
    else if (location_on_wheel < 2000)
        sendPacket(colorPacket((location_on_wheel % 400),0,400));
    else if (location_on_wheel < 2400)
        sendPacket(colorPacket(400,0,400-(location_on_wheel % 400)));
}
